/* Syntactic analyzer */

#include <stdbool.h>
#include <stdarg.h>
#include <stdlib.h>

#include "global_vars.h"
#include "lexical_analyzer.h"
#include "print_helper.h"

#define PRINT_TOKEN 0

/* Syntactic analyzer functions implementation */

enum TokenType tokenType;

static void analyze_TEXT() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == MOT) {
        scanner();
        #if PRINT_TOKEN
        wprint_token();
        #endif
        token[tokenFound].type = tokenTypestr[tokenType];
        tokenFound++;
        analyze_TEXT();
    } else if (tokenType != MOTCLE && tokenType != NPARA && tokenType != SECTION && \
                tokenType != SSECTION && tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_P() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == NPARA) {
        scanner();
        #if PRINT_TOKEN
        wprint_token();
        #endif
        token[tokenFound].type = tokenTypestr[tokenType];
        tokenFound++;
        if (tokenType == MOT) {
            scanner();
            #if PRINT_TOKEN
            wprint_token();
            #endif
            token[tokenFound].type = tokenTypestr[tokenType];
            tokenFound++;
            analyze_TEXT();
            analyze_P();
        } else if (tokenType != SECTION && tokenType != SSECTION && tokenType != FIN) {
            fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
            exit(EXIT_FAILURE);
        }
    } else if (tokenType != SECTION && tokenType != SSECTION && tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_HEAD() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == MOTCLE) {
        scanner();
        #if PRINT_TOKEN
        wprint_token();
        #endif
        token[tokenFound].type = tokenTypestr[tokenType];
        tokenFound++;
        analyze_TEXT();
        if (tokenType == MOTCLE) {
            scanner();
            #if PRINT_TOKEN
            wprint_token();
            #endif /* PRINT_TOKEN */
            token[tokenFound].type = tokenTypestr[tokenType];
            tokenFound++;
            analyze_TEXT();
        } else if (tokenType != NPARA && tokenType != SECTION && tokenType != FIN) {
            fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
            exit(EXIT_FAILURE);
        }
    } else if (tokenType != NPARA && tokenType != SECTION && tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_H1() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SECTION) {
        scanner();
        #if PRINT_TOKEN
        wprint_token();
        #endif /* PRINT_TOKEN */
        token[tokenFound].type = tokenTypestr[tokenType];
        tokenFound++;
        analyze_TEXT();
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_H2() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SSECTION) {
        scanner();
        #if PRINT_TOKEN
        wprint_token();
        #endif /* PRINT_TOKEN */
        token[tokenFound].type = tokenTypestr[tokenType];
        tokenFound++;
        analyze_TEXT();
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_S2() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SSECTION) {
        analyze_H2();
        analyze_P();
        analyze_S2();
    } else if (tokenType != SECTION && tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_S1() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SECTION) {
        analyze_H1();
        analyze_P();
        analyze_S2();
        analyze_S1();
    } else if (tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_BODY() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    analyze_P();
    analyze_S1();
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

void analyze_AXIOME() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    scanner();
    /* print the lexical analysis result */
    #if PRINT_TOKEN
    wprint_token();
    #endif /* PRINT_TOKEN */
    token[tokenFound].type = tokenTypestr[tokenType];
    tokenFound++;
    analyze_HEAD();
    analyze_BODY();
    if (tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "successful syntactic analysis\n");
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}
